@@ -31,7 +31,7 @@
 #include "siproxd.h"
 #include "log.h"
 
-static char const ident[]="$Id: accessctl.c 424 2009-03-28 09:37:47Z hb9xar $";
+static char const ident[]="$Id: accessctl.c 521 2015-09-13 08:43:04Z hb9xar $";
 
 /* configuration storage */
 struct siproxd_config configuration;
@@ -113,7 +113,8 @@ int process_aclist (char *aclist, struct sockaddr_in from) {
    int i, sts;
    int lastentry;
    char *p1, *p2;
-   char address[32]; /* dotted decimal IP - max 15 chars*/
+   char address[HOSTNAME_SIZE+1]; /* dotted decimal IP - max 15 chars */
+                                  /* or hostname*/
    char mask[8];     /* mask - max 2 digits */
    int  mask_int;
    struct in_addr inaddr;
@@ -168,8 +169,12 @@ int process_aclist (char *aclist, struct sockaddr_in from) {
 			    (long)ntohl(from.sin_addr.s_addr) & bitmask);
 
       if ( (ntohl(inaddr.s_addr) & bitmask) == 
-           (ntohl(from.sin_addr.s_addr) & bitmask) ) return STS_SUCCESS;
+           (ntohl(from.sin_addr.s_addr) & bitmask) ) {
+         DEBUGC(DBCLASS_ACCESS, "process_aclist: MATCH");
+         return STS_SUCCESS;
+      }
    }
 
+   DEBUGC(DBCLASS_ACCESS, "process_aclist: no match");
    return STS_FAILURE;
 }
