@@ -19,10 +19,14 @@
 */
 
 #include "config.h"
+#include <string.h>
+#include <stdio.h>
+#include <unistd.h>
 
 #include <sys/types.h>
 #include <netinet/in.h>
 #include <arpa/inet.h>
+#include <sys/socket.h>
 
 #include <osipparser2/osip_parser.h>
 
@@ -30,7 +34,7 @@
 #include "log.h"
 #include "plugins.h"
 
-static char const ident[]="$Id: plugins.c 471 2011-05-28 10:03:49Z hb9xar $";
+static char const ident[]="$Id: plugins.c 499 2014-09-26 21:37:21Z hb9xar $";
 
 /* configuration storage */
 extern struct siproxd_config configuration;
