@@ -37,7 +37,7 @@
 #include "plugins.h"
 #include "log.h"
 
-static char const ident[]="$Id: proxy.c 456 2010-03-29 17:28:12Z hb9xar $";
+static char const ident[]="$Id: proxy.c 528 2015-09-19 22:32:55Z hb9xar $";
 
 /* configuration storage */
 extern struct siproxd_config configuration;	/* defined in siproxd.c */
@@ -79,9 +79,6 @@ int proxy_request (sip_ticket_t *ticket) {
    int i;
    int sts;
    int type;
-   struct in_addr sendto_addr;
-   osip_uri_t *url;
-   int port;
    char *buffer;
    size_t buflen;
    osip_message_t *request;
@@ -117,20 +114,17 @@ int proxy_request (sip_ticket_t *ticket) {
     * by doing a lookup in the registration table.
     */
    sip_find_direction(ticket, &i);
-   type = ticket->direction;
 
    /* Call Plugins for stage: PLUGIN_PRE_PROXY */
    sts = call_plugins(PLUGIN_PRE_PROXY, ticket);
 
+   type = ticket->direction;
    /*
     * RFC 3261, Section 16.6 step 1
     * Proxy Behavior - Request Forwarding - Make a copy
     */
    /* nothing to do here, copy is ready in 'request'*/
 
-   /* get destination address */
-   url=osip_message_get_uri(request);
-
    switch (type) {
   /*
    * from an external host to the internal masqueraded host
@@ -146,7 +140,10 @@ int proxy_request (sip_ticket_t *ticket) {
        * (rewrite request URI to point to the real host)
        */
       /* 'i' still holds the valid index into the URLMAP table */
-      proxy_rewrite_request_uri(request, i);
+DEBUGC(DBCLASS_PROXY,"index i=%i",i);
+      if ((i>=0) && (i < URLMAP_SIZE)) {
+         proxy_rewrite_request_uri(request, i);
+      }
 
       /* if this is CANCEL/BYE request, stop RTP proxying */
       if (MSG_IS_BYE(request) || MSG_IS_CANCEL(request)) {
@@ -197,8 +194,8 @@ sts=sip_obscure_callid(ticket);
 	    request->sip_method? request->sip_method:"*NULL*",
 	    request->from->url->username? request->from->url->username:"*NULL*",
 	    request->from->url->host? request->from->url->host : "*NULL*",
-	    url->username? url->username : "*NULL*",
-	    url->host? url->host : "*NULL*");
+	    request->req_uri->username? request->req_uri->username : "*NULL*",
+	    request->req_uri->host? request->req_uri->host : "*NULL*");
 
          sip_gen_response(ticket, 403 /*forbidden*/);
 
@@ -238,8 +235,8 @@ sts=sip_obscure_callid(ticket);
            request->sip_method? request->sip_method:"*NULL*",
 	   request->from->url->username? request->from->url->username:"*NULL*",
 	   request->from->url->host? request->from->url->host : "*NULL*",
-	   url->username? url->username : "*NULL*",
-	   url->host? url->host : "*NULL*");
+	   request->req_uri->username? request->req_uri->username : "*NULL*",
+	   request->req_uri->host? request->req_uri->host : "*NULL*");
 
 /*
  * we may end up here for two reasons:
@@ -349,30 +346,37 @@ sts=sip_obscure_callid(ticket);
     * Proxy Behavior - Determine Next-Hop Address
     */
 /*&&&& priority probably should be:
- * 1) Route header
- * 2) fixed outbound proxy
- * 3) SIP URI
+ * 1) an already defined next-hop in ticket->next_hop
+ * 2) Route header
+ * 3) fixed outbound proxy
+ * 4) SIP URI
  */
+ 
+   /* predefined next-hop present? */
+   if (is_empty_sockaddr(&ticket->next_hop) == STS_FAILURE) {
+      DEBUGC(DBCLASS_PROXY, "proxy_request: pre-set next-hop: %s:%i",
+             utils_inet_ntoa(ticket->next_hop.sin_addr), ticket->next_hop.sin_port);
    /*
     * Route present?
     * If so, fetch address from topmost Route: header and remove it.
     */
-   if ((type == REQTYP_OUTGOING) && 
+   } else if ((type == REQTYP_OUTGOING) && 
               (!osip_list_eol(&(request->routes), 0))) {
-      sts=route_determine_nexthop(ticket, &sendto_addr, &port);
+      sts=route_determine_nexthop(ticket, &ticket->next_hop.sin_addr, 
+                                  &ticket->next_hop.sin_port);
       if (sts == STS_FAILURE) {
          DEBUGC(DBCLASS_PROXY, "proxy_request: route_determine_nexthop failed");
          return STS_FAILURE;
       }
       DEBUGC(DBCLASS_PROXY, "proxy_request: have Route header to %s:%i",
-             utils_inet_ntoa(sendto_addr), port);
+             utils_inet_ntoa(ticket->next_hop.sin_addr), ticket->next_hop.sin_port);
    /*
     * fixed or domain outbound proxy defined ?
     */
    } else if ((type == REQTYP_OUTGOING) &&
-       (sip_find_outbound_proxy(ticket, &sendto_addr, &port) == STS_SUCCESS)) {
+       (sip_find_outbound_proxy(ticket, &ticket->next_hop.sin_addr, &ticket->next_hop.sin_port) == STS_SUCCESS)) {
       DEBUGC(DBCLASS_PROXY, "proxy_request: have outbound proxy %s:%i",
-             utils_inet_ntoa(sendto_addr), port);
+             utils_inet_ntoa(ticket->next_hop.sin_addr), ticket->next_hop.sin_port);
    /*
     * destination from SIP URI
     */
@@ -383,21 +387,22 @@ In a first implementation we may just try to get the lowest priority,
 max weighted '_sip._udp.domain' entry and port number.
 No load balancing and no failover are supported with this.
 &&&*/
-      sts = get_ip_by_host(url->host, &sendto_addr);
+      sts = get_ip_by_host(request->req_uri->host, &ticket->next_hop.sin_addr);
       if (sts == STS_FAILURE) {
          DEBUGC(DBCLASS_PROXY, "proxy_request: cannot resolve URI [%s]",
-                url->host);
+                request->req_uri->host);
          return STS_FAILURE;
       }
 
-      if (url->port) {
-         port=atoi(url->port);
-         if ((port<=0) || (port>65535)) port=SIP_PORT;
-      } else {
-         port=SIP_PORT;
+      ticket->next_hop.sin_port=SIP_PORT;
+      if (request->req_uri->port) {
+         ticket->next_hop.sin_port=atoi(request->req_uri->port);
+         if (ticket->next_hop.sin_port == 0) {
+            ticket->next_hop.sin_port=SIP_PORT;
+         }
       }
       DEBUGC(DBCLASS_PROXY, "proxy_request: have SIP URI to %s:%i",
-             url->host, port);
+             utils_inet_ntoa(ticket->next_hop.sin_addr), ticket->next_hop.sin_port);
    }
 
    /*
@@ -436,7 +441,8 @@ No load balancing and no failover are supported with this.
       return STS_FAILURE;
    }
 
-   sipsock_send(sendto_addr, port, ticket->protocol, buffer, buflen);
+   sipsock_send(ticket->next_hop.sin_addr, ticket->next_hop.sin_port, 
+                ticket->protocol, buffer, buflen);
    osip_free (buffer);
 
   /*
@@ -473,9 +479,7 @@ No load balancing and no failover are supported with this.
 int proxy_response (sip_ticket_t *ticket) {
    int sts;
    int type;
-   struct in_addr sendto_addr;
    osip_via_t *via;
-   int port;
    char *buffer;
    size_t buflen;
    osip_message_t *response;
@@ -489,6 +493,16 @@ int proxy_response (sip_ticket_t *ticket) {
 
    response=ticket->sipmsg;
 
+   /*
+    * figure out if this is an request coming from the outside
+    * world to one of our registered clients
+    */
+   sip_find_direction(ticket, NULL);
+
+   /* Call Plugins for stage: PLUGIN_PRE_PROXY */
+   sts = call_plugins(PLUGIN_PRE_PROXY, ticket);
+
+   type = ticket->direction;
    /*
     * RFC 3261, Section 16.7 step 3
     * Proxy Behavior - Response Processing - Remove my Via header field value
@@ -500,16 +514,6 @@ int proxy_response (sip_ticket_t *ticket) {
       return STS_FAILURE;
    }
 
-   /*
-    * figure out if this is an request coming from the outside
-    * world to one of our registered clients
-    */
-   sip_find_direction(ticket, NULL);
-   type = ticket->direction;
-
-   /* Call Plugins for stage: PLUGIN_PRE_PROXY */
-   sts = call_plugins(PLUGIN_PRE_PROXY, ticket);
-
 /*
  * ok, we got a response that we are allowed to process.
  */
@@ -667,38 +671,45 @@ sts=sip_obscure_callid(ticket);
     * Determine Next-Hop Address
     */
 /*&&&& priority probably should be:
- * 0) rport=;received= header (TCP only for now)
- * 1) Route header
- * 2) fixed outbound proxy
- * 3) Via header
+ * 1) an already defined next-hop in ticket->next_hop
+ * 2) rport=;received= header (TCP only for now)
+ * 3) Route header
+ * 4) fixed outbound proxy
+ * 5) Via header
  */
+   /* predefined next-hop present? */
+   if (is_empty_sockaddr(&ticket->next_hop) == STS_FAILURE) {
+      DEBUGC(DBCLASS_PROXY, "proxy_request: pre-set next-hop: %s:%i",
+             utils_inet_ntoa(ticket->next_hop.sin_addr), ticket->next_hop.sin_port);
    /*
     * IF TCP, check for rport=x;received=y parameters in VIA
     */
-   if ((ticket->protocol == PROTO_TCP) && 
-       (sip_get_received_param(ticket,  &sendto_addr, &port) == STS_SUCCESS)) {
+   } else if ((ticket->protocol == PROTO_TCP) && 
+       (sip_get_received_param(ticket,  &ticket->next_hop.sin_addr, &ticket->next_hop.sin_port) == STS_SUCCESS)) {
       DEBUGC(DBCLASS_PROXY, "proxy_response: have received/rport to %s:%i",
-             utils_inet_ntoa(sendto_addr), port);
+             utils_inet_ntoa(ticket->next_hop.sin_addr),
+             ticket->next_hop.sin_port);
    /*
     * Route present?
     * If so, fetch address from topmost Route: header and remove it.
     */
    } else if ((type == RESTYP_OUTGOING) && 
               (!osip_list_eol(&(response->routes), 0))) {
-      sts=route_determine_nexthop(ticket, &sendto_addr, &port);
+      sts=route_determine_nexthop(ticket, &ticket->next_hop.sin_addr, &ticket->next_hop.sin_port);
       if (sts == STS_FAILURE) {
          DEBUGC(DBCLASS_PROXY, "proxy_response: route_determine_nexthop failed");
          return STS_FAILURE;
       }
       DEBUGC(DBCLASS_PROXY, "proxy_response: have Route header to %s:%i",
-             utils_inet_ntoa(sendto_addr), port);
+             utils_inet_ntoa(ticket->next_hop.sin_addr),
+             ticket->next_hop.sin_port);
    /*
     * check if we need to send to an outbound proxy
     */
    } else if ((type == RESTYP_OUTGOING) &&
-       (sip_find_outbound_proxy(ticket, &sendto_addr, &port) == STS_SUCCESS)) {
+       (sip_find_outbound_proxy(ticket, &ticket->next_hop.sin_addr, &ticket->next_hop.sin_port) == STS_SUCCESS)) {
       DEBUGC(DBCLASS_PROXY, "proxy_response: have outbound proxy %s:%i",
-             utils_inet_ntoa(sendto_addr), port);
+             utils_inet_ntoa(ticket->next_hop.sin_addr), ticket->next_hop.sin_port);
    } else {
       /* get target address and port from VIA header */
       via = (osip_via_t *) osip_list_get (&(response->vias), 0);
@@ -707,18 +718,19 @@ sts=sip_obscure_callid(ticket);
          return STS_FAILURE;
       }
 
-      sts = get_ip_by_host(via->host, &sendto_addr);
+      sts = get_ip_by_host(via->host, &ticket->next_hop.sin_addr);
       if (sts == STS_FAILURE) {
          DEBUGC(DBCLASS_PROXY, "proxy_response: cannot resolve VIA [%s]",
                 via->host);
          return STS_FAILURE;
       }
 
+      ticket->next_hop.sin_port=SIP_PORT;
       if (via->port) {
-         port=atoi(via->port);
-         if ((port<=0) || (port>65535)) port=SIP_PORT;
-      } else {
-         port=SIP_PORT;
+         ticket->next_hop.sin_port=atoi(via->port);
+         if (ticket->next_hop.sin_port == 0) {
+            ticket->next_hop.sin_port=ticket->next_hop.sin_port;
+         }
       }
    }
 
@@ -734,7 +746,7 @@ sts=sip_obscure_callid(ticket);
       return STS_FAILURE;
    }
 
-   sipsock_send(sendto_addr, port, ticket->protocol, buffer, buflen);
+   sipsock_send(ticket->next_hop.sin_addr, ticket->next_hop.sin_port, ticket->protocol, buffer, buflen);
    osip_free (buffer);
    return STS_SUCCESS;
 }
@@ -1153,7 +1165,7 @@ if (configuration.debuglevel)
    sprintf(clen,"%ld",(long)buflen);
    sts = osip_message_set_content_length(mymsg, clen);
 
-   /* free old body */
+   /* free new body string*/
    osip_free(buff);
 
 if (configuration.debuglevel)
@@ -1185,11 +1197,10 @@ if (configuration.debuglevel)
  *	STS_SUCCESS on success
  */
 int proxy_rewrite_request_uri(osip_message_t *mymsg, int idx){
-   char *scheme;
-   char *username;
-   char *host;
-   char *port;
    osip_uri_t *url;
+   int sts;
+   char *tmp1=NULL;
+   char *tmp2=NULL;
 
    if ((idx >= URLMAP_SIZE) || (idx < 0)) {
       WARN("proxy_rewrite_request_uri: called with invalid index");
@@ -1199,51 +1210,19 @@ int proxy_rewrite_request_uri(osip_message_t *mymsg, int idx){
    DEBUGC(DBCLASS_PROXY,"rewriting incoming Request URI");
    url=osip_message_get_uri(mymsg);
 
-   /* set the true scheme */
-   if (url->scheme) {osip_free(url->scheme);url->scheme=NULL;}
-   if (urlmap[idx].true_url->scheme) {
-      DEBUGC(DBCLASS_BABBLE,"proxy_rewrite_request_uri: scheme=%s",
-             urlmap[idx].true_url->scheme);
-      scheme = (char *)osip_malloc(strlen(urlmap[idx].true_url->scheme)+1);
-      memcpy(scheme, urlmap[idx].true_url->scheme,
-             strlen(urlmap[idx].true_url->scheme));
-      scheme[strlen(urlmap[idx].true_url->scheme)]='\0';
-      osip_uri_set_scheme(url, scheme);
-   }
+   osip_uri_to_str(url, &tmp1);
+   osip_uri_to_str(urlmap[idx].true_url, &tmp2);
+   DEBUGC(DBCLASS_BABBLE,"proxy_rewrite_request_uri: %s -> %s", tmp1, tmp2);
 
-   /* set the true username */
-   if (url->username) {osip_free(url->username);url->username=NULL;}
-   if (urlmap[idx].true_url->username) {
-      DEBUGC(DBCLASS_BABBLE,"proxy_rewrite_request_uri: username=%s",
-             urlmap[idx].true_url->username);
-      username = (char*)osip_malloc(strlen(urlmap[idx].true_url->username)+1);
-      memcpy(username, urlmap[idx].true_url->username,
-             strlen(urlmap[idx].true_url->username));
-      username[strlen(urlmap[idx].true_url->username)]='\0';
-      osip_uri_set_username(url, username);
+   osip_uri_free(url);
+   url=NULL;
+   osip_message_set_uri(mymsg, url);
+   sts = osip_uri_clone(urlmap[idx].true_url, &url);
+   if (sts != 0) {
+      ERROR("osip_uri_clone failed");
    }
+   osip_message_set_uri(mymsg, url);
 
-   /* set the true host */
-   if (url->host) {osip_free(url->host);url->host=NULL;}
-   if (urlmap[idx].true_url->host) {
-      DEBUGC(DBCLASS_BABBLE,"proxy_rewrite_request_uri: host=%s",
-             urlmap[idx].true_url->host);
-      host = (char *)osip_malloc(strlen(urlmap[idx].true_url->host)+1);
-      memcpy(host, urlmap[idx].true_url->host, strlen(urlmap[idx].true_url->host));
-      host[strlen(urlmap[idx].true_url->host)]='\0';
-      osip_uri_set_host(url, host);
-   }
-
-   /* set the true port */
-   if (url->port) {osip_free(url->port);url->port=NULL;}
-   if (urlmap[idx].true_url->port) {
-      DEBUGC(DBCLASS_BABBLE,"proxy_rewrite_request_uri: port=%s",
-             urlmap[idx].true_url->port);
-      port = (char *)osip_malloc(strlen(urlmap[idx].true_url->port)+1);
-      memcpy(port, urlmap[idx].true_url->port, strlen(urlmap[idx].true_url->port));
-      port[strlen(urlmap[idx].true_url->port)]='\0';
-      osip_uri_set_port(url, port);
-   }
    return STS_SUCCESS;
 }
 
