@@ -42,7 +42,7 @@
 #include "digcalc.h"
 #include "log.h"
 
-static char const ident[]="$Id: sip_utils.c 467 2011-01-09 13:10:27Z hb9xar $";
+static char const ident[]="$Id: sip_utils.c 538 2016-02-24 19:14:23Z hb9xar $";
 
 
 /* configuration storage */
@@ -270,18 +270,20 @@ int compare_url(osip_uri_t *url1, osip_uri_t *url2) {
       return STS_FAILURE;
    }
 
-   DEBUGC(DBCLASS_PROXY, "comparing urls: %s:%s@%s -> %s:%s@%s",
+#if 0
+   DEBUGC(DBCLASS_BABBLE, "comparing urls: %s:%s@%s -> %s:%s@%s",
          (url1->scheme)   ? url1->scheme :   "(null)",
          (url1->username) ? url1->username : "(null)",
          (url1->host)     ? url1->host :     "(null)",
          (url2->scheme)   ? url2->scheme :   "(null)",
          (url2->username) ? url2->username : "(null)",
          (url2->host)     ? url2->host :     "(null)");
+#endif
 
    /* compare SCHEME (if present) case INsensitive */
    if (url1->scheme && url2->scheme) {
       if (osip_strcasecmp(url1->scheme, url2->scheme) != 0) {
-         DEBUGC(DBCLASS_PROXY, "compare_url: scheme mismatch");
+         DEBUGC(DBCLASS_BABBLE, "compare_url: scheme mismatch");
          return STS_FAILURE;
       }
    } else {
@@ -291,11 +293,11 @@ int compare_url(osip_uri_t *url1, osip_uri_t *url2) {
    /* compare username (if present) case sensitive */
    if (url1->username && url2->username) {
       if (strcmp(url1->username, url2->username) != 0) {
-         DEBUGC(DBCLASS_PROXY, "compare_url: username mismatch");
+//         DEBUGC(DBCLASS_BABBLE, "compare_url: username mismatch");
          return STS_FAILURE;
       }
    } else {
-      DEBUGC(DBCLASS_PROXY, "compare_url: NULL username - ignoring");
+//      DEBUGC(DBCLASS_BABBLE, "compare_url: NULL username - ignoring");
    }
 
 
@@ -308,26 +310,26 @@ int compare_url(osip_uri_t *url1, osip_uri_t *url2) {
    /* get the IP addresses from the (possible) hostnames */
    sts1=get_ip_by_host(url1->host, &addr1);
    if (sts1 == STS_FAILURE) {
-      DEBUGC(DBCLASS_PROXY, "compare_url: cannot resolve host [%s]",
+      DEBUGC(DBCLASS_BABBLE, "compare_url: cannot resolve host [%s]",
              url1->host);
    }
 
    sts2=get_ip_by_host(url2->host, &addr2);
    if (sts2 == STS_FAILURE) {
-      DEBUGC(DBCLASS_PROXY, "compare_url: cannot resolve host [%s]",
+      DEBUGC(DBCLASS_BABBLE, "compare_url: cannot resolve host [%s]",
              url2->host);
    }
 
    if ((sts1 == STS_SUCCESS) && (sts2 == STS_SUCCESS)) {
       /* compare IP addresses */
       if (memcmp(&addr1, &addr2, sizeof(addr1))!=0) {
-         DEBUGC(DBCLASS_PROXY, "compare_url: IP mismatch");
+         DEBUGC(DBCLASS_BABBLE, "compare_url: IP mismatch");
          return STS_FAILURE;
       }
    } else {
       /* compare hostname strings case INsensitive */
       if (osip_strcasecmp(url1->host, url2->host) != 0) {
-         DEBUGC(DBCLASS_PROXY, "compare_url: host name mismatch");
+         DEBUGC(DBCLASS_BABBLE, "compare_url: host name mismatch");
          return STS_FAILURE;
       }
    }
@@ -661,6 +663,10 @@ int sip_rewrite_contact (sip_ticket_t *ticket, int direction) {
       if (contact == NULL) break;
       if (contact->url == NULL) continue;
 
+      DEBUGC(DBCLASS_PROXY, "trying to rewriting Contact header %s@%s",
+             (contact->url->username)? contact->url->username : "*NULL*",
+             (contact->url->host)? contact->url->host : "*NULL*");
+
       /* search for an entry */
       for (i=0;i<URLMAP_SIZE;i++){
          if (urlmap[i].active == 0) continue;
@@ -711,6 +717,32 @@ int sip_rewrite_contact (sip_ticket_t *ticket, int direction) {
 
          osip_list_add(&(sip_msg->contacts),contact,j);
          replaced=1;
+      } else {
+#if 1
+         /* FALLBACK - some devices send crappy Contact headers that
+          * are completely different from the Contact used in REGISTER.
+          * This is a hack-around, trying *somehow* to produce a "meaningful"
+          * masqueraded contact header (at least it does point back to the proxy
+          * and not into some RFC1918 network).
+          */
+         if (direction == DIR_OUTGOING) {
+            DEBUGC(DBCLASS_PROXY, "outgoing: no match - fallback to plain proxy IP");
+            struct in_addr addr;
+            if (get_interface_ip(IF_OUTBOUND, &addr) == STS_SUCCESS) {
+               char *myaddr;
+               myaddr=utils_inet_ntoa(addr);
+
+               /* Host */
+               osip_free(contact->url->host);
+               contact->url->host = osip_strdup(myaddr);
+               /* Port */
+               contact->url->port=realloc(contact->url->port, 16);
+               sprintf(contact->url->port, "%i", configuration.sip_listen_port);
+
+               replaced=1;
+            }
+         } // if direction
+#endif
       }
 
    } /* for j */
@@ -883,7 +915,7 @@ int  sip_calculate_branch_id (sip_ticket_t *ticket, char *id) {
  *	STS_FAILURE if no outbound proxy to be used
  */
 int  sip_find_outbound_proxy(sip_ticket_t *ticket, struct in_addr *addr,
-                             int *port) {
+                             in_port_t *port) {
    int i, sts;
    char *domain=NULL;
    osip_message_t *sipmsg;
@@ -931,7 +963,7 @@ int  sip_find_outbound_proxy(sip_ticket_t *ticket, struct in_addr *addr,
                return STS_FAILURE;
             }
             *port=atoi(configuration.outbound_proxy_domain_port.string[i]);
-            if ((*port<=0) || (*port>65535)) *port=SIP_PORT;
+//            if ((*port<=0) || (*port>65535)) *port=SIP_PORT;
 
             return STS_SUCCESS;
 
@@ -967,7 +999,7 @@ int  sip_find_outbound_proxy(sip_ticket_t *ticket, struct in_addr *addr,
 
 
 /*
- * SIP_IS_OUTGOING
+ * SIP_FIND_DIRECTION
  *
  * Figures out if this is an outgoing or incoming request/response.
  * The direction is stored in the ticket->direction property.
@@ -991,15 +1023,22 @@ int  sip_find_direction(sip_ticket_t *ticket, int *urlidx) {
 
    ticket->direction = DIRTYP_UNKNOWN;
 
+   DEBUGC(DBCLASS_SIP, "sip_find_direction: beginning search");
+
    /* Search order is as follows:
-    * - "from" IP address one of our registered local UAs
-    * - "To:" SIP header (Request) directed to internal UA
+    * - "from" IP address is one of our registered local UAs
+    *   => OUTGOING
+    * - "Req: To:" SIP header (Request) directed to internal UA
     *   or
-    *   "From:" SIP header (Response) coming from internal UA
-    * - SIP URI matches one of the registered local UAs
-    * - for Responses: check for bottommost "Via:" header to be
+    *   Resp: "From:" SIP header (Response) coming from internal UA
+    *   => INCOMING
+    * - Req: SIP URI matches one of the registered local UAs
+    *   => INCOMING
+    * - Resp: check for bottommost "Via:" header to be
     *   one of our registered local UA IPs
-    * - "from" IP == 127.0.0.1 || inbound_IP || outbound IP 
+    *   => INCOMING
+    * - "from" IP == (127.0.0.1 | inbound_IP | outbound IP)
+    *   => OUTGOING
     *
     * The first successful match is taken.
     */
@@ -1014,8 +1053,8 @@ int  sip_find_direction(sip_ticket_t *ticket, int *urlidx) {
          DEBUGC(DBCLASS_SIP, "sip_find_direction: cannot resolve host [%s]",
              urlmap[i].true_url->host);
       } else {
-         DEBUGC(DBCLASS_SIP, "sip_find_direction: reghost:%s ip:%s",
-                urlmap[i].true_url->host, utils_inet_ntoa(from->sin_addr));
+//         DEBUGC(DBCLASS_BABBLE, "sip_find_direction: reghost:%s ip:%s",
+//                urlmap[i].true_url->host, utils_inet_ntoa(from->sin_addr));
          if (memcmp(&tmp_addr, &from->sin_addr, sizeof(tmp_addr)) == 0) {
             if (MSG_IS_REQUEST(ticket->sipmsg)) {
                type=REQTYP_OUTGOING;
@@ -1026,6 +1065,9 @@ int  sip_find_direction(sip_ticket_t *ticket, int *urlidx) {
          }
       }
    }
+   if (type == DIRTYP_UNKNOWN) {
+      DEBUGC(DBCLASS_SIP, "sip_find_direction: no OUTGOING found");
+   }
 
 
    /*
@@ -1068,6 +1110,9 @@ int  sip_find_direction(sip_ticket_t *ticket, int *urlidx) {
          } /* is request */
       } /* for i */
    } /* if type == DIRTYP_UNKNOWN */
+   if (type == DIRTYP_UNKNOWN) {
+      DEBUGC(DBCLASS_SIP, "sip_find_direction: no INCOMING (To:) found");
+   }
 
 
    /*
@@ -1087,6 +1132,9 @@ int  sip_find_direction(sip_ticket_t *ticket, int *urlidx) {
          }
       } /* for i */
    } /* if type == DIRTYP_UNKNOWN */
+   if (type == DIRTYP_UNKNOWN) {
+      DEBUGC(DBCLASS_SIP, "sip_find_direction: no INCOMING RQ (SIP URI) found");
+   }
 
 
    /* &&&& Open Issue &&&&
@@ -1094,26 +1142,36 @@ int  sip_find_direction(sip_ticket_t *ticket, int *urlidx) {
     * (e.g 1393xxx@proxy01.sipphone.com for calls made sipphone -> FWD)
     * How can we deal with this? Should I take into consideration the 'Via'
     * headers? This is the only clue I have, pointing to the *real* UA.
-    * Maybe I should put in a 'siproxd' ftag value to recognize it as a header
+    * Maybe I should put in a 'siproxd' flag value to recognize it as a header
     * inserted by myself
     */
    if ((type == DIRTYP_UNKNOWN) && 
-       (!osip_list_eol(&(response->vias), 0))) {
+       (!osip_list_eol(&(response->vias), 1))) {
       if (MSG_IS_RESPONSE(ticket->sipmsg)) {
          osip_via_t *via;
          struct in_addr addr_via, addr_myself;
          int port_via, port_ua;
 
-         /* get the via address */
-         via = (osip_via_t *) osip_list_get (&(response->vias), 0);
-         DEBUGC(DBCLASS_SIP, "sip_find_direction: check via [%s] for "
-                "registered UA",via->host);
-         sts=get_ip_by_host(via->host, &addr_via);
-         if (sts == STS_FAILURE) {
-            DEBUGC(DBCLASS_SIP, "sip_find_direction: cannot resolve VIA [%s]",
-                   via->host);
+         /* get the via address :
+          * topmost via (pos 0) still is my own via (not yet removed)
+          * next vi a(pos 1) may be local UA via
+          */
+         via = (osip_via_t *) osip_list_get (&(response->vias), 1);
+         if (via) {
+            DEBUGC(DBCLASS_SIP, "sip_find_direction: check via [%s] for "
+                   "registered UA",via->host);
+            sts=get_ip_by_host(via->host, &addr_via);
+            if (sts == STS_FAILURE) {
+               DEBUGC(DBCLASS_SIP, "sip_find_direction: cannot resolve VIA [%s]",
+                      via->host);
+            }
          } else {
+            DEBUGC(DBCLASS_SIP, "sip_find_direction: no second via in list");
+            sts = STS_FAILURE;
+         }
 
+
+         if (sts == STS_SUCCESS) {
             for (i=0; i<URLMAP_SIZE; i++) {
                if (urlmap[i].active == 0) continue;
                /* incoming response (1st via in list points to a registered UA) */
@@ -1133,7 +1191,7 @@ int  sip_find_direction(sip_ticket_t *ticket, int *urlidx) {
                   port_ua=atoi(urlmap[i].true_url->port);
                if ((port_ua<=0) || (port_ua>65535)) port_ua=SIP_PORT;
 
-               DEBUGC(DBCLASS_SIP, "sip_find_direction: checking for registered "
+               DEBUGC(DBCLASS_BABBLE, "sip_find_direction: checking for registered "
                       "host [%s:%i] <-> [%s:%i]",
                       urlmap[i].true_url->host, port_ua,
                       via->host, port_via);
@@ -1147,6 +1205,9 @@ int  sip_find_direction(sip_ticket_t *ticket, int *urlidx) {
          }
       } /* is response */
    } /* if type == DIRTYP_UNKNOWN */
+   if (type == DIRTYP_UNKNOWN) {
+      DEBUGC(DBCLASS_SIP, "sip_find_direction: no INCOMING RS (via header) found");
+   }
 
 
    /*
@@ -1172,16 +1233,32 @@ int  sip_find_direction(sip_ticket_t *ticket, int *urlidx) {
 	 }
       }
    } /* if type == DIRTYP_UNKNOWN */
-
+   if (type == DIRTYP_UNKNOWN) {
+      DEBUGC(DBCLASS_SIP, "sip_find_direction: no INCOMING (redirected) found");
+   }
+   
 
    if (type == DIRTYP_UNKNOWN) {
       DEBUGC(DBCLASS_SIP, "sip_find_direction: unable to determine "
                           "direction of SIP packet");
+      if (urlidx) *urlidx=-1;
       return STS_FAILURE;
    }
 
    ticket->direction=type;
-   if (urlidx) *urlidx=i;
+
+   if (i < URLMAP_SIZE) {
+      if (urlidx) *urlidx=i;
+      DEBUGC(DBCLASS_SIP, "sip_find_direction: dir=%i, urlmap %i, "
+                          "trueurl [%s:%s] / masqurl [%s:%s] / regurl [%s:%s]",
+                          type, i,
+                          urlmap[i].true_url->host, urlmap[i].true_url->port,
+                          urlmap[i].masq_url->host, urlmap[i].masq_url->port,
+                          urlmap[i].reg_url->host, urlmap[i].reg_url->port);
+   } else {
+      if (urlidx) *urlidx=-1;
+      DEBUGC(DBCLASS_SIP, "sip_find_direction: dir=%i, not found in URLMAP", type);
+   }
    return STS_SUCCESS;
 }
 
@@ -1423,7 +1500,7 @@ int sip_add_received_param(sip_ticket_t *ticket){
  *	STS_SUCCESS on success
  */
 int  sip_get_received_param(sip_ticket_t *ticket,
-                            struct in_addr *dest, int *port) {
+                            struct in_addr *dest, in_port_t *port) {
    osip_via_t *via;
    osip_generic_param_t *received=NULL;
    osip_generic_param_t *rport=NULL;
@@ -1442,7 +1519,7 @@ int  sip_get_received_param(sip_ticket_t *ticket,
       
       /* fetch the port number */
       *port = atoi(rport->gvalue);
-      if ((*port <=0) || (*port >=65536)) return STS_FAILURE;
+//      if ((*port <=0) || (*port >=65536)) return STS_FAILURE;
 
       /* If TCP, then validate first if an existing connection is in the cache.
        * If not, do not use this - a new conection must be established! */
