@@ -50,7 +50,7 @@
 #include "siproxd.h"
 #include "log.h"
 
-static char const ident[]="$Id: utils.c 481 2011-06-12 17:53:48Z hb9xar $";
+static char const ident[]="$Id: utils.c 536 2016-02-03 20:02:27Z hb9xar $";
 
 /* configuration storage */
 extern struct siproxd_config configuration;
@@ -94,6 +94,14 @@ int get_ip_by_host(char *hostname, struct in_addr *addr) {
       return STS_FAILURE;
    }
 
+   /* check if passed string is already a plain IPv4 address */
+   if (utils_inet_aton(hostname, addr) > 0) {
+      /* is a plain IPv4 string - return the result directly, no lookup and cache */ 
+//      DEBUGC(DBCLASS_BABBLE, "DNS lookup - shortcut, hostname is IP string [%s] -> [%s]",
+//             hostname, utils_inet_ntoa(*addr));
+      return STS_SUCCESS;
+   }
+
    /* first time: initialize DNS cache */
    if (cache_initialized == 0) {
       DEBUGC(DBCLASS_DNS, "initializing DNS cache (%i entries)", DNS_CACHE_SIZE);
@@ -130,7 +138,7 @@ int get_ip_by_host(char *hostname, struct in_addr *addr) {
             idx=i;
             break;
          }
-         DEBUGC(DBCLASS_DNS, "DNS lookup - from cache: %s -> %s",
+         DEBUGC(DBCLASS_BABBLE, "DNS lookup - from cache: %s -> %s",
                 hostname, utils_inet_ntoa(*addr));
          return STS_SUCCESS;
       }
@@ -607,7 +615,7 @@ char *utils_inet_ntoa(struct in_addr in) {
  * implements an inet_aton()
  *
  * converts the string in *cp and stores it into inp
- * Returns != 0 on success
+ * Returns > 0 on success
  */
 int  utils_inet_aton(const char *cp, struct in_addr *inp) {
 #if defined(HAVE_INET_PTON)
@@ -678,3 +686,17 @@ int  compare_client_id(client_id_t cid1, client_id_t cid2) {
    DEBUGC(DBCLASS_BABBLE, "compare_client_id: no match");
    return STS_FAILURE;
 }
+
+
+/*
+ * check a sockaddr_in structure for bein zero / non-zero
+ */
+int  is_empty_sockaddr(struct sockaddr_in *sockaddr) {
+   int i;
+   char *p=(char*)sockaddr;
+
+   for (i=0; i < sizeof(struct sockaddr_in); i++) {
+      if (p[i] != 0x00) { return STS_FAILURE; }
+   }
+   return STS_SUCCESS;
+}
